#include "mainwindow.h"
#include <QApplication>
#include <QString>
#include <QScreen>
#include <QDebug>
#include <QJsonDocument>
#include <QJsonObject>

bool readJson(QString filename, InputParameter& input)
{
    QString json;
    QFile file;
    QJsonValue object_value;

    qInfo() << "Configuration parameters from: " << filename;

    file.setFileName(filename);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        qCritical() << "Configuration file \"" << filename << "\" does not exist.";
        return false;
    }
    json = file.readAll();
    file.close();

    QJsonDocument document = QJsonDocument::fromJson(json.toUtf8());
    QJsonObject object = document.object();

    object_value = object.value(QString("isFileInput"));
    if(object_value == QJsonValue::Undefined)
    {
        qCritical() << "Invalid configuration file content format.\n";
        return false;
    }
    input.isFileInput = object_value.toBool();

    object_value = object.value(QString("isSimulation"));
    if(object_value == QJsonValue::Undefined)
    {
        input.isSimulation = false;
    }
    input.isFileInput = object_value.toBool();

    object_value = object.value(QString("isFileOutput"));
    if(object_value == QJsonValue::Undefined)
    {
        qCritical() << "Invalid configuration file content format.\n";
        return false;
    }
    input.isFileOutput = object_value.toBool();

    if(input.isFileInput)
    {
        object_value = object.value(QString("inputFileName"));
        if(object_value == QJsonValue::Undefined)
        {
            qCritical() << "Invalid configuration file content format.\n";
            return false;
        }
        input.inputFileName = object_value.toString();
        qInfo() << "Input get from file: " << input.inputFileName;
    }
    else
    {
        object_value = object.value(QString("inputGStreamer"));
        if(object_value == QJsonValue::Undefined)
        {
            qCritical() << "Invalid configuration file content format.\n";
            return false;
        }
        input.inputGStreamer = object_value.toString();
        qInfo() << "Input get from stream: " <<  input.inputGStreamer;
    }

    if(input.isFileOutput)
    {
        object_value = object.value(QString("outputFileName"));
        if(object_value == QJsonValue::Undefined)
        {
            qCritical() << "Invalid configuration file content format.\n";
            return false;
        }
        input.outputFileName = object_value.toString();
        qInfo() << "Output saved in file: " <<  input.outputFileName;
    }

    object_value = object.value(QString("scale"));
    if(object_value == QJsonValue::Undefined)
    {
        input.scale = 6;
    }
    else
    {
        input.scale = object_value.toInt();
    }

    object_value = object.value(QString("targetFps"));
    if(object_value == QJsonValue::Undefined)
    {
        object_value = object.value(QString("targetFPS"));
        if(object_value == QJsonValue::Undefined)
        {
            input.targetFps = 30;
        }
        else
        {
            input.targetFps = object_value.toInt();
            qInfo() << "Target FPS set to " << input.targetFps;
        }
    }
    else
    {        
        input.targetFps = object_value.toInt();
        qInfo() << "Target FPS set to " << input.targetFps;
    }

    object_value = object.value(QString("polynomiumDegree"));
    if(object_value == QJsonValue::Undefined)
    {
        input.polynomium_degree = 1;
    }
    else
    {
        input.polynomium_degree = object_value.toInt();
        qInfo() << "Polynomium degree set to " << input.polynomium_degree;
    }

    object_value = object.value(QString("centerPoints"));
    if(object_value == QJsonValue::Undefined)
    {
        input.center_points = 200;
    }
    else
    {
        input.center_points = object_value.toInt();
        qInfo() << "Center points set to " << input.center_points;
    }

    object_value = object.value(QString("inputDelay"));
    if(object_value == QJsonValue::Undefined)
    {
        input.inputDelay = 12;
    }
    else
    {
        input.inputDelay = object_value.toInt();
        qInfo() << "Input delay set to " << input.inputDelay;
    }

    object_value = object.value(QString("medianFilterSize"));
    if(object_value == QJsonValue::Undefined)
    {
        input.medianFilterSize = 3;
    }
    else
    {
        input.medianFilterSize = object_value.toInt();
    }

    object_value = object.value(QString("temporalFilterOffset"));
    if(object_value == QJsonValue::Undefined)
    {
        input.temporalFilterOffset = 3;
    }
    else
    {
        input.temporalFilterOffset = object_value.toInt();
    }

    object_value = object.value(QString("temporalFilterThreshold"));
    if(object_value == QJsonValue::Undefined)
    {
        input.temporalFilterThreshold = 10;
    }
    else
    {
        input.temporalFilterThreshold = object_value.toInt();
    }

    object_value = object.value(QString("m"));
    if(object_value == QJsonValue::Undefined)
    {
        input.m = 6;
    }
    else
    {
        input.m = static_cast<float>(object_value.toDouble());
    }

    object_value = object.value(QString("threshold"));
    if(object_value == QJsonValue::Undefined)
    {
        input.threshold = 20;
    }
    else
    {
        input.threshold = static_cast<float>(object_value.toDouble());
    }

    object_value = object.value(QString("minVal"));
    if(object_value == QJsonValue::Undefined)
    {
        input.minVal = 20;
    }
    else
    {
        input.minVal = static_cast<float>(object_value.toDouble());
    }

    object_value = object.value(QString("mFilter"));
    if(object_value == QJsonValue::Undefined)
    {
        input.mFilter = 20;
    }
    else
    {
        input.mFilter = static_cast<float>(object_value.toDouble());
    }

    object_value = object.value(QString("height"));
    if(object_value == QJsonValue::Undefined)
    {
        input.height = static_cast<float>(1.220);
    }
    else
    {
        input.height = static_cast<float>(object_value.toDouble());
    }

    object_value = object.value(QString("pitch"));
    if(object_value == QJsonValue::Undefined)
    {
        input.pitch = 3;
    }
    else
    {
        input.pitch = static_cast<float>(object_value.toDouble());
    }

    object_value = object.value(QString("distAhead"));
    if(object_value == QJsonValue::Undefined)
    {
        input.distAhead = 40;
    }
    else
    {
        input.distAhead = static_cast<float>(object_value.toDouble());
    }

    object_value = object.value(QString("spaceToOneSide"));
    if(object_value == QJsonValue::Undefined)
    {
        input.spaceToOneSide = static_cast<float>(3.3);
    }
    else
    {
        input.spaceToOneSide = static_cast<float>(object_value.toDouble());
    }

    object_value = object.value(QString("bottomOffset"));
    if(object_value == QJsonValue::Undefined)
    {
        input.bottomOffset = 7;
    }
    else
    {
        input.bottomOffset = static_cast<float>(object_value.toDouble());
    }

    return true;
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);    

    QSize size = app.screens()[0]->size();
    bool is_desktop = false;

    if(argc != 2)
    {
        if(argc == 3)
        {
            if(std::strcmp(argv[2], "desktop") == 0)
            {
                is_desktop = true;
            }
        }
        else
        {
            qCritical() << "Wrong parameters number, expected configuration file name.\n";
            return -1;
        }
    }

    if(!is_desktop)
    {
        if(!QDir("/home/root/GOLDLaneDetector").exists())
        {
            QDir().mkdir("/home/root/GOLDLaneDetector");
        }
        if(!QDir("/home/root/GOLDLaneDetector/input").exists())
        {
            QDir().mkdir("/home/root/GOLDLaneDetector/input");
        }
        if(!QDir("/home/root/GOLDLaneDetector/output").exists())
        {
            QDir().mkdir("/home/root/GOLDLaneDetector/output");
        }
        if(!QDir("/home/root/GOLDLaneDetector/log").exists())
        {
            QDir().mkdir("/home/root/GOLDLaneDetector/log");
        }
    }

    struct sched_param param;
    pthread_setschedparam(pthread_self(), SCHED_RR, &param);

    if(is_desktop)
        qInfo() << "Is desktop: TRUE";
    else
        qInfo() << "Is desktop: FALSE";

    QString filename(argv[1]);

    InputParameter input;
    if(!readJson(filename, input))
    {
        return -1;
    }

    input.isDesktop = is_desktop;

    MainWindow main_window(size);
    main_window.initializeThread(input);
    main_window.show();

    return app.exec();
}


