#include "mainwindow.h"

MainWindow::MainWindow(QSize size, QWidget *parent) : QWidget(parent)
{
    /* Window */
    if(size.width() * 9 <= size.height() * 16)
    {        
        setFixedSize(size.width(), size.width() * 9 / 16);
        this->size = QSize(size.width(), size.width() * 9 / 16);
    }
    else
    {
        setFixedSize(size.height() * 16 / 9, size.height());
        this->size = QSize(size.height() * 16 / 9, size.height());
    }   

    setVisible(true);
    setBackgroundRole(QPalette::Light);

    main_label = new QLabel();
    main_label->setGeometry(0, 0, size.width(), size.height());
    main_label->setVisible(true);
}

MainWindow::~MainWindow()
{
    delete main_label;
}

void MainWindow::error()
{
    QApplication::exit(-1);
}

void MainWindow::quit()
{
    std::cout << "The application has exited correctly." << std::endl;
    sleep(2);
    QApplication::exit(0);
}

void MainWindow::update_content(QPixmap image)
{
    image = image.scaled(size);
    main_label->setPixmap(image);
}

void MainWindow::initializeThread(InputParameter input)
{
    execution_thread = new QThread();

    worker = new GOLDWorker(input);
    worker->moveToThread(execution_thread);

    QObject::connect(execution_thread, SIGNAL (started()), worker, SLOT (process()));
    QObject::connect(worker, SIGNAL(request_new_content(QPixmap)), this, SLOT(update_content(QPixmap)));
    QObject::connect(worker, SIGNAL (finished()), execution_thread, SLOT (quit()));
    QObject::connect(worker, SIGNAL (finished()), worker, SLOT (deleteLater()));
    QObject::connect(worker, SIGNAL (finished()), this, SLOT (quit()));
    QObject::connect(execution_thread, SIGNAL (finished()), execution_thread, SLOT (deleteLater()));
    QObject::connect(worker, SIGNAL (error(QString)), worker, SLOT (deleteLater()));
    QObject::connect(worker, SIGNAL (error(QString)), execution_thread, SLOT (quit()));
    QObject::connect(worker, SIGNAL (error(QString)), this, SLOT (error()));

    execution_thread->start();
}
