function [transformation_matrix, image_to_vehicle_matrix, outSize] = generate_transformation_matrix(focalLength, principalPoint, imageSize, ...
    height, pitch, distAhead, spaceToOneSide, bottomOffset, outWidth)
    
    camIntrinsics = cameraIntrinsics(focalLength,principalPoint,imageSize);

    cameraObject = monoCamera(camIntrinsics,height,'Pitch',pitch);
    outView = [bottomOffset,distAhead,-spaceToOneSide,spaceToOneSide];
    outImageSize = [NaN, outWidth];
    birdsEye = birdsEyeView(cameraObject,outView,outImageSize);

    adjTform = ...
        [0, -1,  0;  % -1 here causes y to become x and reverses the axis direction
        -1,  0,  0;  % x becomes y and reverses axis direction
         0,  0,  1]; % no translation
    bevTform = birdsEye.Sensor.ImageToVehicleTransform.T*adjTform;

    reqImgHW = outImageSize;
    nanIdxHW = isnan(reqImgHW);
    worldHW  = abs([outView(2)-outView(1), outView(4)-outView(3)]);
    scale   = (reqImgHW(~nanIdxHW)-1)/worldHW(~nanIdxHW);
    scaleXY = [scale, scale];

    outDimFrac = scale*worldHW(nanIdxHW);
    otherDim   = round(outDimFrac)+1;

    outSize = cast(reqImgHW, 'like', otherDim);
    outSize(nanIdxHW) = otherDim;

    dYdXVehicle = [outView(4), outView(2)];
    tXY = scaleXY.* dYdXVehicle; % X,Y translation in pixels

    viewMatrix = ...
        [scaleXY(1),          0,  0;
        0          , scaleXY(2),  0;
        tXY(1)+1   , tXY(2)+1  ,  1]; % +1 for placing origin at (1,1)

    transformation_matrix = bevTform*viewMatrix;
    
    image_to_vehicle_matrix = birdsEye.ImageToVehicleTransform.T;
end