clearvars
close all
clc

%% Inputs

W = 1920;
H = 1080;
scale = 2;
input_image = imread('input.png');
grey_image = rgb2gray(input_image); %convert from rgb to greyscale

focalLength = [2.015422939513609e+03 2.002217072647680e+03];
principalPoint = [W/2 H/2];
imageSize = [H W];
height = 1.220;
pitch = 2.5; %pitch toward the ground
distAhead = 40; 
spaceToOneSide = 3.3;
bottomOffset = 7;
outWidth = 150;

threshold = 10;
m = 5; 
min_val = 10;
m_filter = 10;

[transformation_matrix, image_to_vehicle_matrix, outSize] = generate_transformation_matrix(focalLength, principalPoint, imageSize, ...
    height, pitch, distAhead, spaceToOneSide, bottomOffset, outWidth);

%% Pre-processing
dest = birds_eye_view(grey_image, transformation_matrix, outSize);
%dest = imwarp(grey_image, transformation_matrix, 'o', imref2d(outSize));
    
%% Processing
[p, filtered] = filter_grey_bev_1080p(dest, m, threshold, min_val, m_filter, image_to_vehicle_matrix);
disp(p);

%% Post-processing
%filtered = imread('ex1.jpg');

[transformation_matrix_inverse, outSize] = generate_transformation_matrix_inverse(focalLength, principalPoint, imageSize, ...
    height, pitch, distAhead, spaceToOneSide, bottomOffset, outWidth);

bev_image_size = [H/scale W/scale];

inverse_image = birds_eye_view_inv(filtered, transformation_matrix_inverse, scale, bev_image_size);
inverse_image = imresize(inverse_image, scale);

%% Prints
% figure(1)
% title('Original Image')
% imshow(input_image);
% 
% figure(2)
% 
% subplot(1,2,1);
% title('Transformed Image')
% imshow(dest);
% subplot(1,2,2);
% title('Transformed Image')
% imshow(filtered);
% 
% figure(4)
% title('Inverse Image')
% imshow(inverse_image);
% % 
% figure(5)
% image(grey_image)
% axis image 
% hold on
% 
% im = image(inverse_image);
% im.AlphaData = max(inverse_image,[],3); 
% hold off
